////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                        //
//  LibSocketCAN header translation by Thomas Kuhn                                                        //
//  More info can be found at https://lalten.github.io/libsocketcan/Documentation/html/group__extern.html //
//                                                                                                        //
////////////////////////////////////////////////////////////////////////////////////////////////////////////

unit can.lib;

{$mode objfpc}
{$packrecords c}
{$H+}

interface

(* libsocketcan.c
 *
 * (C) 2009 Luotao Fu <l.fu@pengutronix.de>
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *)

uses
  ctypes, can.netlink;

{$linklib c}
{$linklib libsocketcan}

const
  CAN_LIB_SUCCESS =  0; // for function call results
  CAN_LIB_FAILED  = -1; // for function call results

type

  // from <linux/if_link.h>
  rtnl_link_stats64 = record
    rx_packets:          cuint64;
    tx_packets:          cuint64;
    rx_bytes:            cuint64;
    tx_bytes:            cuint64;
    rx_errors:           cuint64;
    tx_errors:           cuint64;
    rx_dropped:          cuint64;
    tx_dropped:          cuint64;
    multicast:           cuint64;
    collisions:          cuint64;

    // detailed rx_errors:
    rx_length_errors:    cuint64;
    rx_over_errors:      cuint64;
    rx_crc_errors:       cuint64;
    rx_frame_errors:     cuint64;
    rx_fifo_errors:      cuint64;
    rx_missed_errors:    cuint64;

    // detailed tx_errors:
    tx_aborted_errors:   cuint64;
    tx_carrier_errors:   cuint64;
    tx_fifo_errors:      cuint64;
    tx_heartbeat_errors: cuint64;
    tx_window_errors:    cuint64;

    // for cslip etc
    rx_compressed:       cuint64;
    tx_compressed:       cuint64;
    rx_nohandler:        cuint64;
  end;

  pcan_bittiming       = ^can_bittiming;
  pcan_ctrlmode        = ^can_ctrlmode;
  pcan_clock           = ^can_clock;
  pcan_bittiming_const = ^can_bittiming_const;
  pcan_berr_counter    = ^can_berr_counter;
  pcan_device_stats    = ^can_device_stats;
  prtnl_link_stats64   = ^rtnl_link_stats64;

  // These functions work only with root rights
  function can_do_restart(const myifname: ansistring): cint; cdecl; external;
  function can_set_restart_ms(const name: ansistring; restart_ms: cuint32): cint; cdecl; external;
  function can_do_stop(const name: ansistring): cint; cdecl; external;
  function can_do_start(const name: ansistring): cint; cdecl; external;
  function can_set_bitrate(const name: ansistring; bitrate: cuint32): cint; cdecl; external; // To use this function, make sure that CONFIG_CAN_CALC_BITTIMING is set to y in your kernel configuration
  function can_set_bitrate_samplepoint(const name: ansistring; bitrate, samplepoint: cuint32): cint; cdecl; external;
  function can_set_bittiming(const name: ansistring; bt: pcan_bittiming): cint; cdecl; external;
  function can_set_ctrlmode(const name: ansistring; cm: pcan_ctrlmode): cint; cdecl; external;

  // These functions work with user rights
  function can_get_restart_ms(const name: ansistring; restart_ms: puint32): cint; cdecl; external;
  function can_get_bittiming(const name: ansistring; bt: pcan_bittiming): cint; cdecl; external;
  function can_get_ctrlmode(const name: ansistring; cm: pcan_ctrlmode): cint; cdecl; external;
  function can_get_state(const name: ansistring; state: pcint): cint; cdecl; external;
  function can_get_clock(const name: ansistring; clock: pcan_clock): cint; cdecl; external;
  function can_get_bittiming_const(const name: ansistring; btc: pcan_bittiming_const): cint; cdecl; external;
  function can_get_berr_counter(const name: ansistring; bc: pcan_berr_counter): cint; cdecl; external;
  function can_get_device_stats(const name: ansistring; cds: pcan_device_stats): cint; cdecl; external;
  function can_get_link_stats(const name: ansistring; rls: prtnl_link_stats64): cint; cdecl; external;


implementation


end.

