unit excersise;

{$mode delphi}
{$H+}

interface

uses
  Classes, SysUtils, Forms, StdCtrls, can.lib, can.netlink, ctypes;

type

  { TLibScketCanForm }

  TLibScketCanForm = class(TForm)
    B_GetCtrlMode: TButton;
    B_GetClock: TButton;
    B_GetDevStats: TButton;
    B_GetLinkStats: TButton;
    B_GetResms: TButton;
    B_GetBittimingC: TButton;
    B_GetBerrCounter: TButton;
    B_GetState: TButton;
    B_GetBitTime: TButton;
    B_Stop: TButton;
    B_Start: TButton;
    B_SetBaud: TButton;
    CBbaudrate: TComboBox;
    CanInterfaceName: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label_CtrlMode: TLabel;
    Label_State: TLabel;
    Label_BitTime: TLabel;
    Label_DevStats: TLabel;
    Label_LinkStats: TLabel;
    Label_Restartms: TLabel;
    Label_BerrCount: TLabel;
    Label_BittimingC: TLabel;
    Label_Clock: TLabel;
    Label_StopResult: TLabel;
    Label_StartResult: TLabel;
    Label_BaudrateResult: TLabel;
    procedure B_GetBitTimeClick(Sender: TObject);
    procedure B_GetBittimingCClick(Sender: TObject);
    procedure B_GetCtrlModeClick(Sender: TObject);
    procedure B_GetClockClick(Sender: TObject);
    procedure B_GetBerrCounterClick(Sender: TObject);
    procedure B_GetDevStatsClick(Sender: TObject);
    procedure B_GetLinkStatsClick(Sender: TObject);
    procedure B_GetResmsClick(Sender: TObject);
    procedure B_GetStateClick(Sender: TObject);
    procedure B_SetBaudClick(Sender: TObject);
    procedure B_StopClick(Sender: TObject);
    procedure B_StartClick(Sender: TObject);
    procedure CanInterfaceNameChange(Sender: TObject);
    procedure CBbaudrateChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private

  public

  end;

const
  common_bitrates: array[0..10] of cuint32 = (
     1000000,
      800000,
      500000,
      250000,
      125000,
      100000,
       83300, // SWCAN high speed (SAE J2411, GMLAN)
       50000,
       33300, // SWCAN low speed  (SAE J2411, GMLAN)
       20000,
       10000);

var
  LibScketCanForm: TLibScketCanForm;
  //ifr_name       : ansistring;
  myBaudrateIndex: longint;

implementation

{$R *.lfm}

function CanResultToStr(ACanResult: cint): string;
begin
  case ACanResult of
    CAN_LIB_SUCCESS: Result := 'Success(';
    CAN_LIB_FAILED:  Result := 'Failed(';
  else
    Result := 'Undefined('
  end;
  Result := Result + ACanResult.ToString + ')'
end;

function CanStateToStr(ACanState: cint): string;
begin
  case ACanState of
    CAN_STATE_ERROR_ACTIVE:  Result := 'CAN state error active';
    CAN_STATE_ERROR_WARNING: Result := 'CAN state error warning';
    CAN_STATE_ERROR_PASSIVE: Result := 'CAN state error passive';
    CAN_STATE_BUS_OFF:       Result := 'CAN state bus off';
    CAN_STATE_STOPPED:       Result := 'CAN state stopped';
    CAN_STATE_SLEEPING:      Result := 'CAN state sleeping';
  else
    Result := 'CAN state undefined'
  end;
end;

{ TLibScketCanForm }

procedure TLibScketCanForm.B_GetCtrlModeClick(Sender: TObject);
var
  myresult:      cint;
  myControlMode: can_ctrlmode;
begin
  myresult := can_get_ctrlmode(CanInterfaceName.Text, @myControlMode);
  Label_CtrlMode.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_CtrlMode.Caption := Label_CtrlMode.Caption + ', ' + ' mask:$' + IntToHex(myControlMode.mask) + ' flags:$' + IntToHex(myControlMode.flags);
end;

procedure TLibScketCanForm.B_GetClockClick(Sender: TObject);
var
  myresult: cint;
  myclock : can_clock;
begin
  myresult := can_get_clock(CanInterfaceName.Text, @myclock);
  Label_Clock.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_Clock.Caption := Label_Clock.Caption + ', freq:' + myclock.freq.ToString;
end;

procedure TLibScketCanForm.B_GetBerrCounterClick(Sender: TObject);
var
  myresult     : cint;
  myBerrCounter: can_berr_counter;
begin
  myresult := can_get_berr_counter(CanInterfaceName.Text, @myBerrCounter);
  Label_BerrCount.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_BerrCount.Caption := Label_BerrCount.Caption + ', txerr:' + myBerrCounter.txerr.ToString + ' rxerr:' + myBerrCounter.rxerr.ToString;
end;

procedure TLibScketCanForm.B_GetDevStatsClick(Sender: TObject);
var
  myresult     : cint;
  myDeviceStats: can_device_stats;
begin
  myresult := can_get_device_stats(CanInterfaceName.Text, @myDeviceStats);
  Label_DevStats.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_DevStats.Caption := Label_DevStats.Caption + ', be:' + myDeviceStats.bus_error.ToString        + ' ew:' + myDeviceStats.error_warning.ToString
                                                     +  ' ep:' + myDeviceStats.error_passive.ToString    + ' bo:' + myDeviceStats.bus_off.ToString
                                                     +  ' al:' + myDeviceStats.arbitration_lost.ToString + ' rs:' + myDeviceStats.restarts.ToString;
end;

procedure TLibScketCanForm.B_GetLinkStatsClick(Sender: TObject);
var
  myresult   : cint;
  myLinkStats: rtnl_link_stats64;
begin
  myresult := can_get_link_stats(CanInterfaceName.Text, @myLinkStats);
  Label_LinkStats.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_LinkStats.Caption := Label_LinkStats.Caption + ', rxp:' + myLinkStats.rx_packets.ToString + ' txp:' + myLinkStats.tx_packets.ToString
                                                       +  ' rxb:' + myLinkStats.rx_bytes.ToString   + ' txb:' + myLinkStats.tx_bytes.ToString
                                                       +  ' rxe:' + myLinkStats.rx_errors.ToString  + ' txe:' + myLinkStats.tx_errors.ToString
                                                       +  ' rxd:' + myLinkStats.rx_dropped.ToString + ' txd:' + myLinkStats.tx_dropped.ToString
                                                       +  ' mul:' + myLinkStats.multicast.ToString  + ' col:' + myLinkStats.collisions.ToString;
end;

procedure TLibScketCanForm.B_GetResmsClick(Sender: TObject);
var
  myresult: cint;
  myresms : cuint32;
begin
  myresult := can_get_restart_ms(CanInterfaceName.Text, @myresms);
  Label_Restartms.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_Restartms.Caption := Label_Restartms.Caption + ', Restartms:' + myresms.ToString;
end;

procedure TLibScketCanForm.B_StopClick(Sender: TObject);
var
  myresult: cint;
begin
  myresult := can_do_stop(CanInterfaceName.Text);
  Label_StopResult.Caption := CanResultToStr(myresult);
end;

procedure TLibScketCanForm.B_StartClick(Sender: TObject);
var
  myresult: cint;
begin
  myresult := can_do_start(CanInterfaceName.Text);
  Label_StartResult.Caption := CanResultToStr(myresult);
end;

procedure TLibScketCanForm.CanInterfaceNameChange(Sender: TObject);
begin
  CanInterfaceName.Text := Trim(CanInterfaceName.Text);
end;

procedure TLibScketCanForm.CBbaudrateChange(Sender: TObject);
begin
  myBaudrateIndex := CBbaudrate.ItemIndex;
end;

procedure TLibScketCanForm.FormShow(Sender: TObject);
begin
  LibScketCanForm.ActiveControl := CanInterfaceName;
end;

procedure TLibScketCanForm.B_GetStateClick(Sender: TObject);
var
  myresult, mystate : cint;
begin
  myresult := can_get_state(CanInterfaceName.Text, @mystate);
  Label_State.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_State.Caption := Label_State.Caption + ', ' + CanStateToStr(mystate);
end;

procedure TLibScketCanForm.B_SetBaudClick(Sender: TObject);
var
  myresult  : cint;
  mybaudrate: cuint32;
begin
  mybaudrate := common_bitrates[myBaudrateIndex];
  myresult   := can_set_bitrate(CanInterfaceName.Text, mybaudrate);
  Label_BaudrateResult.Caption := CanResultToStr(myresult);
end;

procedure TLibScketCanForm.B_GetBitTimeClick(Sender: TObject);
var
  myresult   : cint;
  myBittiming: can_bittiming;
begin
  myresult := can_get_bittiming(CanInterfaceName.Text, @myBittiming);
  Label_BitTime.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_BitTime.Caption := Label_BitTime.Caption + ', br:' + InttoStr(myBittiming.bitrate)    + ' sp:'  + InttoStr(myBittiming.sample_point)
                                                   +  ' tq:' + InttoStr(myBittiming.tq)         + ' sp:'  + InttoStr(myBittiming.prop_seg)
                                                   + ' ph1:' + InttoStr(myBittiming.phase_seg1) + ' h2:'  + InttoStr(myBittiming.phase_seg2)
                                                   +  ' sj:' + InttoStr(myBittiming.bitrate)    + ' brp:' + InttoStr(myBittiming.sample_point);
end;

procedure TLibScketCanForm.B_GetBittimingCClick(Sender: TObject);
var
  myresult        : cint;
  myBittimingconst: can_bittiming_const;
begin
  myresult := can_get_bittiming_const(CanInterfaceName.Text, @myBittimingconst);
  Label_BittimingC.Caption := CanResultToStr(myresult);
  if myresult = CAN_LIB_SUCCESS then
    Label_BittimingC.Caption := Label_BittimingC.Caption + ', ts1-:' + myBittimingconst.tseg1_min.ToString + ' ts1+:' + myBittimingconst.tseg1_max.ToString
                                                         +  ' ts2-:' + myBittimingconst.tseg2_min.ToString + ' ts2+:' + myBittimingconst.tseg2_max.ToString
                                                         +  ' sjw+:' + myBittimingconst.sjw_max.ToString   + ' brp-:' + myBittimingconst.brp_min.ToString
                                                         +  ' brp+:' + myBittimingconst.brp_max.ToString   + ' brpi:' + myBittimingconst.brp_inc.ToString
                                                         +  ' name:' + StrPas(myBittimingconst.name);
end;

end.
